/*
* 07/08/2003 - 19:09:03
*
* TSTGenerator.java - 
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package searchassist.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import com.wallyflint.util.TernarySearchTree;

/**
 * @author Ashwin Jayaprakash
 */
public class TSTGenerator extends DefaultHandler
{
	private ObjectOutputStream oos;
	private TernarySearchTree keywords;
	private String currTagName;
	private String currUniqueID;

	private static final boolean DEBUG = true;

	public TSTGenerator(ObjectOutputStream objOS, TernarySearchTree tst)
	{
		oos = objOS;
		keywords = tst;
	} //end constr

	//===========================================================
	// SAX DocumentHandler methods
	//===========================================================
	public void startElement(String namespaceURI, String lName, // local name
	String qName, // qualified name
	Attributes attrs) throws SAXException
	{
		String eName = lName; // element name
		if ("".equals(eName))
		{
			eName = qName; // namespaceAware = false
		} //end if
		currTagName = eName;

		boolean lookForUniqueID = doesTagContainKeyWordsTag(eName);

		if (attrs != null)
		{
			for (int i = 0; i < attrs.getLength(); i++)
			{
				String aName = attrs.getLocalName(i); // Attr name 
				if ("".equals(aName))
				{
					aName = attrs.getQName(i);
					if (lookForUniqueID && aName.equals(getUniqueIDAttrName()))
					{
						currUniqueID = attrs.getValue(i);
					} //end inner if
				} //end if
			} //end for
		} //end outer if
	}

	public void characters(char buf[], int offset, int len) throws SAXException
	{
		String s = new String(buf, offset, len).trim();
		if (!s.equals(""))
		{
			if (currTagName.equals(getTagNameContainingKeywords()))
			{

				StringTokenizer tokenizer = new StringTokenizer(s, ",");
				String token;

				while (tokenizer.hasMoreTokens())
				{
					token = tokenizer.nextToken();
					List listOfUniqueIDs = keywords.get(token);

					if (listOfUniqueIDs != null)
					{
						boolean addThisKeyword = false;

						for (Iterator iter = listOfUniqueIDs.iterator(); iter.hasNext();)
						{
							String element = (String) iter.next();
							//Check if parent is already there.
							if (element.length() < currUniqueID.length() && currUniqueID.startsWith(element))
							{
								break;
							} //end innermost if
							//Check if unique ID is already there.
							else if (element.equals(currUniqueID))
							{
								break;
							} //end innermost else if
							//Check if child is already there.
							else if (currUniqueID.length() < element.length() && element.startsWith(currUniqueID))
							{
								iter.remove();
								addThisKeyword = true;
								break;
							} //end innermost else if
							//Unrelated nodes in the list.
							else
							{
                                addThisKeyword = true;
                                break;
							} //end innermost else
						} //end for

						if (addThisKeyword)
						{
							listOfUniqueIDs.add(currUniqueID);
							keywords.put(token, listOfUniqueIDs);
						} //end innermost if
					} //end inner if
					else
					{
						listOfUniqueIDs = new LinkedList();
						listOfUniqueIDs.add(currUniqueID);
						keywords.put(token, listOfUniqueIDs);
					} //end inner  else
				} //end while
			} //end if
		} //end outer if
	} //end method

	public void endDocument() throws SAXException
	{
		try
		{
			oos.writeObject(keywords);
			oos.flush();
			oos.close();
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	//===========================================================
	// SAX ErrorHandler methods
	//===========================================================
	// treat validation errors as fatal
	public void error(SAXParseException e) throws SAXParseException
	{
		throw e;
	} // dump warnings too

	public void warning(SAXParseException err) throws SAXParseException
	{
		System.out.println("** Warning" + ", line " + err.getLineNumber() + ", uri " + err.getSystemId());
		System.out.println("   " + err.getMessage());
	}

	protected String getTagNameContainingKeywords()
	{
		return "keywords";
	} //end method

	protected boolean doesTagContainKeyWordsTag(String tagName)
	{
		boolean retValue = false;

		if (tagName.equals("category") || tagName.equals("item"))
		{
			retValue = true;
		} //end if
		return retValue;
	} //end method

	protected String getUniqueIDAttrName()
	{
		return "uniqueID";
	} //end method

	public static void main(String argv[])
	{
		if (argv.length < 2)
		{
			System.err.println("Usage: cmd input_filename ser_tst_filename");
			System.exit(1);
		}

		try
		{
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(new File(argv[1])));

			// Use an instance of ourselves as the SAX event handler
			TernarySearchTree tst = new TernarySearchTree();
			DefaultHandler handler = new TSTGenerator(oos, tst);
			// Use the validating parser
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(false);
			//                        factory.setNamespaceAware(true);
			// Parse the input
			SAXParser saxParser = factory.newSAXParser();
			XMLReader xmlReader = saxParser.getXMLReader();
			saxParser.parse(new File(argv[0]), handler);
		}
		catch (SAXParseException spe)
		{
			// Error generated by the parser
			System.out.println("\n** Parsing error" + ", line " + spe.getLineNumber() + ", uri " + spe.getSystemId());
			System.out.println("   " + spe.getMessage());
			// Use the contained exception, if any
			Exception x = spe;
			if (spe.getException() != null)
			{
				x = spe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (SAXException sxe)
		{
			// Error generated by this application
			// (or a parser-initialization error)
			Exception x = sxe;
			if (sxe.getException() != null)
			{
				x = sxe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (ParserConfigurationException pce)
		{ // Parser with specified options can't be built
			pce.printStackTrace();
		}
		catch (IOException ioe)
		{ // I/O error
			ioe.printStackTrace();
		}

		System.exit(0);
	} //end main
} //end class
